import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { localizeWithPath, localize2WithPath } from 'vscode/vscode/vs/nls';
import { KeyChord } from 'vscode/vscode/vs/base/common/keyCodes';
import { ModesRegistry } from 'vscode/vscode/vs/editor/common/languages/modesRegistry';
import { Registry } from 'vscode/vscode/vs/platform/registry/common/platform';
import { registerAction2, Action2, MenuId, MenuRegistry } from 'vscode/vscode/vs/platform/actions/common/actions';
import { OUTPUT_MODE_ID, OUTPUT_MIME, LOG_MODE_ID, LOG_MIME, OUTPUT_VIEW_ID, IOutputService, Extensions as Extensions$3, CONTEXT_IN_OUTPUT, CONTEXT_OUTPUT_SCROLL_LOCK, CONTEXT_ACTIVE_FILE_OUTPUT, ACTIVE_OUTPUT_CHANNEL_CONTEXT } from 'vscode/vscode/vs/workbench/services/output/common/output';
import { OutputViewPane } from './outputView.js';
import { SyncDescriptor } from 'vscode/vscode/vs/platform/instantiation/common/descriptors';
import { Extensions as Extensions$1 } from 'vscode/vscode/vs/workbench/common/contributions';
import { Extensions } from 'vscode/vscode/vs/workbench/common/views';
import { IViewsService } from 'vscode/vscode/vs/workbench/services/views/common/viewsService';
import { ViewPaneContainer } from 'vscode/vscode/vs/workbench/browser/parts/views/viewPaneContainer';
import { Extensions as Extensions$2 } from 'vscode/vscode/vs/platform/configuration/common/configurationRegistry';
import { IQuickInputService } from 'vscode/vscode/vs/platform/quickinput/common/quickInput';
import { AUX_WINDOW_GROUP, IEditorService } from 'vscode/vscode/vs/workbench/services/editor/common/editorService';
import { assertIsDefined } from 'vscode/vscode/vs/base/common/types';
import { ContextKeyExpr } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { Codicon } from 'vscode/vscode/vs/base/common/codicons';
import { registerIcon } from 'vscode/vscode/vs/platform/theme/common/iconRegistry';
import { Categories } from 'vscode/vscode/vs/platform/action/common/actionCommonCategories';
import { Disposable, toDisposable, dispose } from 'vscode/vscode/vs/base/common/lifecycle';
import { IFilesConfigurationService } from 'vscode/vscode/vs/workbench/services/filesConfiguration/common/filesConfigurationService';
import { IAudioCueService, AudioCue } from 'vscode/vscode/vs/platform/audioCues/browser/audioCueService';

ModesRegistry.registerLanguage({
    id: OUTPUT_MODE_ID,
    extensions: [],
    mimetypes: [OUTPUT_MIME]
});
ModesRegistry.registerLanguage({
    id: LOG_MODE_ID,
    extensions: [],
    mimetypes: [LOG_MIME]
});
const outputViewIcon = registerIcon('output-view-icon', Codicon.output, ( localizeWithPath(
    'vs/workbench/contrib/output/browser/output.contribution',
    'outputViewIcon',
    'View icon of the output view.'
)));
const VIEW_CONTAINER = ( Registry.as(Extensions.ViewContainersRegistry)).registerViewContainer({
    id: OUTPUT_VIEW_ID,
    title: ( localize2WithPath(
        'vs/workbench/contrib/output/browser/output.contribution',
        'output',
        "Output"
    )),
    icon: outputViewIcon,
    order: 1,
    ctorDescriptor: ( new SyncDescriptor(
        ViewPaneContainer,
        [OUTPUT_VIEW_ID, { mergeViewWithContainerWhenSingleView: true }]
    )),
    storageId: OUTPUT_VIEW_ID,
    hideIfEmpty: true,
}, 1 , { doNotRegisterOpenCommand: true });
( Registry.as(Extensions.ViewsRegistry)).registerViews([{
        id: OUTPUT_VIEW_ID,
        name: ( localize2WithPath(
            'vs/workbench/contrib/output/browser/output.contribution',
            'output',
            "Output"
        )),
        containerIcon: outputViewIcon,
        canMoveView: true,
        canToggleVisibility: false,
        ctorDescriptor: ( new SyncDescriptor(OutputViewPane)),
        openCommandActionDescriptor: {
            id: 'workbench.action.output.toggleOutput',
            mnemonicTitle: ( localizeWithPath(
                'vs/workbench/contrib/output/browser/output.contribution',
                { key: 'miToggleOutput', comment: ['&& denotes a mnemonic'] },
                "&&Output"
            )),
            keybindings: {
                primary: 2048  | 1024  | 51 ,
                linux: {
                    primary: KeyChord(2048  | 41 , 2048  | 38 )
                }
            },
            order: 1,
        }
    }], VIEW_CONTAINER);
let OutputContribution = class OutputContribution extends Disposable {
    constructor(outputService, editorService, fileConfigurationService) {
        super();
        this.outputService = outputService;
        this.editorService = editorService;
        this.fileConfigurationService = fileConfigurationService;
        this.registerActions();
    }
    registerActions() {
        this.registerSwitchOutputAction();
        this.registerShowOutputChannelsAction();
        this.registerClearOutputAction();
        this.registerToggleAutoScrollAction();
        this.registerOpenActiveOutputFileAction();
        this.registerOpenActiveOutputFileInAuxWindowAction();
        this.registerShowLogsAction();
        this.registerOpenLogFileAction();
    }
    registerSwitchOutputAction() {
        this._register(registerAction2(class extends Action2 {
            constructor() {
                super({
                    id: `workbench.output.action.switchBetweenOutputs`,
                    title: ( localizeWithPath(
                        'vs/workbench/contrib/output/browser/output.contribution',
                        'switchBetweenOutputs.label',
                        "Switch Output"
                    )),
                });
            }
            async run(accessor, channelId) {
                if (channelId) {
                    accessor.get(IOutputService).showChannel(channelId, true);
                }
            }
        }));
        const switchOutputMenu = ( new MenuId('workbench.output.menu.switchOutput'));
        this._register(MenuRegistry.appendMenuItem(MenuId.ViewTitle, {
            submenu: switchOutputMenu,
            title: ( localizeWithPath(
                'vs/workbench/contrib/output/browser/output.contribution',
                'switchToOutput.label',
                "Switch Output"
            )),
            group: 'navigation',
            when: ( ContextKeyExpr.equals('view', OUTPUT_VIEW_ID)),
            order: 1,
            isSelection: true
        }));
        const registeredChannels = ( new Map());
        this._register(toDisposable(() => dispose(( registeredChannels.values()))));
        const registerOutputChannels = (channels) => {
            for (const channel of channels) {
                const title = channel.label;
                const group = channel.extensionId ? '0_ext_outputchannels' : '1_core_outputchannels';
                registeredChannels.set(channel.id, registerAction2(class extends Action2 {
                    constructor() {
                        super({
                            id: `workbench.action.output.show.${channel.id}`,
                            title,
                            toggled: ( ACTIVE_OUTPUT_CHANNEL_CONTEXT.isEqualTo(channel.id)),
                            menu: {
                                id: switchOutputMenu,
                                group,
                            }
                        });
                    }
                    async run(accessor) {
                        return accessor.get(IOutputService).showChannel(channel.id, true);
                    }
                }));
            }
        };
        registerOutputChannels(this.outputService.getChannelDescriptors());
        const outputChannelRegistry = ( Registry.as(Extensions$3.OutputChannels));
        this._register(outputChannelRegistry.onDidRegisterChannel(e => {
            const channel = this.outputService.getChannelDescriptor(e);
            if (channel) {
                registerOutputChannels([channel]);
            }
        }));
        this._register(outputChannelRegistry.onDidRemoveChannel(e => {
            registeredChannels.get(e)?.dispose();
            registeredChannels.delete(e);
        }));
    }
    registerShowOutputChannelsAction() {
        this._register(registerAction2(class extends Action2 {
            constructor() {
                super({
                    id: 'workbench.action.showOutputChannels',
                    title: ( localize2WithPath(
                        'vs/workbench/contrib/output/browser/output.contribution',
                        'showOutputChannels',
                        "Show Output Channels..."
                    )),
                    category: ( localize2WithPath(
                        'vs/workbench/contrib/output/browser/output.contribution',
                        'output',
                        "Output"
                    )),
                    f1: true
                });
            }
            async run(accessor) {
                const outputService = accessor.get(IOutputService);
                const quickInputService = accessor.get(IQuickInputService);
                const extensionChannels = [], coreChannels = [];
                for (const channel of outputService.getChannelDescriptors()) {
                    if (channel.extensionId) {
                        extensionChannels.push(channel);
                    }
                    else {
                        coreChannels.push(channel);
                    }
                }
                const entries = [];
                for (const { id, label } of extensionChannels) {
                    entries.push({ id, label });
                }
                if (extensionChannels.length && coreChannels.length) {
                    entries.push({ type: 'separator' });
                }
                for (const { id, label } of coreChannels) {
                    entries.push({ id, label });
                }
                const entry = await quickInputService.pick(entries, { placeHolder: ( localizeWithPath(
                    'vs/workbench/contrib/output/browser/output.contribution',
                    'selectOutput',
                    "Select Output Channel"
                )) });
                if (entry) {
                    return outputService.showChannel(entry.id);
                }
            }
        }));
    }
    registerClearOutputAction() {
        this._register(registerAction2(class extends Action2 {
            constructor() {
                super({
                    id: `workbench.output.action.clearOutput`,
                    title: ( localize2WithPath(
                        'vs/workbench/contrib/output/browser/output.contribution',
                        'clearOutput.label',
                        "Clear Output"
                    )),
                    category: Categories.View,
                    menu: [{
                            id: MenuId.ViewTitle,
                            when: ( ContextKeyExpr.equals('view', OUTPUT_VIEW_ID)),
                            group: 'navigation',
                            order: 2
                        }, {
                            id: MenuId.CommandPalette
                        }, {
                            id: MenuId.EditorContext,
                            when: CONTEXT_IN_OUTPUT
                        }],
                    icon: Codicon.clearAll
                });
            }
            async run(accessor) {
                const outputService = accessor.get(IOutputService);
                const audioCueService = accessor.get(IAudioCueService);
                const activeChannel = outputService.getActiveChannel();
                if (activeChannel) {
                    activeChannel.clear();
                    audioCueService.playAudioCue(AudioCue.clear);
                }
            }
        }));
    }
    registerToggleAutoScrollAction() {
        this._register(registerAction2(class extends Action2 {
            constructor() {
                super({
                    id: `workbench.output.action.toggleAutoScroll`,
                    title: ( localize2WithPath(
                        'vs/workbench/contrib/output/browser/output.contribution',
                        'toggleAutoScroll',
                        "Toggle Auto Scrolling"
                    )),
                    tooltip: ( localizeWithPath(
                        'vs/workbench/contrib/output/browser/output.contribution',
                        'outputScrollOff',
                        "Turn Auto Scrolling Off"
                    )),
                    menu: {
                        id: MenuId.ViewTitle,
                        when: ( ContextKeyExpr.and(( ContextKeyExpr.equals('view', OUTPUT_VIEW_ID)))),
                        group: 'navigation',
                        order: 3,
                    },
                    icon: Codicon.lock,
                    toggled: {
                        condition: CONTEXT_OUTPUT_SCROLL_LOCK,
                        icon: Codicon.unlock,
                        tooltip: ( localizeWithPath(
                            'vs/workbench/contrib/output/browser/output.contribution',
                            'outputScrollOn',
                            "Turn Auto Scrolling On"
                        ))
                    }
                });
            }
            async run(accessor) {
                const outputView = accessor.get(IViewsService).getActiveViewWithId(OUTPUT_VIEW_ID);
                outputView.scrollLock = !outputView.scrollLock;
            }
        }));
    }
    registerOpenActiveOutputFileAction() {
        const that = this;
        this._register(registerAction2(class extends Action2 {
            constructor() {
                super({
                    id: `workbench.action.openActiveLogOutputFile`,
                    title: ( localize2WithPath(
                        'vs/workbench/contrib/output/browser/output.contribution',
                        'openActiveOutputFile',
                        "Open Output in Editor"
                    )),
                    menu: [{
                            id: MenuId.ViewTitle,
                            when: ( ContextKeyExpr.equals('view', OUTPUT_VIEW_ID)),
                            group: 'navigation',
                            order: 4,
                            isHiddenByDefault: true
                        }],
                    icon: Codicon.goToFile,
                    precondition: CONTEXT_ACTIVE_FILE_OUTPUT
                });
            }
            async run() {
                that.openActiveOutoutFile();
            }
        }));
    }
    registerOpenActiveOutputFileInAuxWindowAction() {
        const that = this;
        this._register(registerAction2(class extends Action2 {
            constructor() {
                super({
                    id: `workbench.action.openActiveLogOutputFileInNewWindow`,
                    title: ( localize2WithPath(
                        'vs/workbench/contrib/output/browser/output.contribution',
                        'openActiveOutputFileInNewWindow',
                        "Open Output in New Window"
                    )),
                    menu: [{
                            id: MenuId.ViewTitle,
                            when: ( ContextKeyExpr.equals('view', OUTPUT_VIEW_ID)),
                            group: 'navigation',
                            order: 5,
                            isHiddenByDefault: true
                        }],
                    icon: Codicon.emptyWindow,
                    precondition: CONTEXT_ACTIVE_FILE_OUTPUT
                });
            }
            async run() {
                that.openActiveOutoutFile(AUX_WINDOW_GROUP);
            }
        }));
    }
    async openActiveOutoutFile(group) {
        const fileOutputChannelDescriptor = this.getFileOutputChannelDescriptor();
        if (fileOutputChannelDescriptor) {
            await this.fileConfigurationService.updateReadonly(fileOutputChannelDescriptor.file, true);
            await this.editorService.openEditor({
                resource: fileOutputChannelDescriptor.file,
                options: {
                    pinned: true,
                },
            }, group);
        }
    }
    getFileOutputChannelDescriptor() {
        const channel = this.outputService.getActiveChannel();
        if (channel) {
            const descriptor = this.outputService.getChannelDescriptors().filter(c => c.id === channel.id)[0];
            if (descriptor?.file) {
                return descriptor;
            }
        }
        return null;
    }
    registerShowLogsAction() {
        this._register(registerAction2(class extends Action2 {
            constructor() {
                super({
                    id: 'workbench.action.showLogs',
                    title: ( localize2WithPath(
                        'vs/workbench/contrib/output/browser/output.contribution',
                        'showLogs',
                        "Show Logs..."
                    )),
                    category: Categories.Developer,
                    menu: {
                        id: MenuId.CommandPalette,
                    },
                });
            }
            async run(accessor) {
                const outputService = accessor.get(IOutputService);
                const quickInputService = accessor.get(IQuickInputService);
                const extensionLogs = [], logs = [];
                for (const channel of outputService.getChannelDescriptors()) {
                    if (channel.log) {
                        if (channel.extensionId) {
                            extensionLogs.push(channel);
                        }
                        else {
                            logs.push(channel);
                        }
                    }
                }
                const entries = [];
                for (const { id, label } of logs) {
                    entries.push({ id, label });
                }
                if (extensionLogs.length && logs.length) {
                    entries.push({ type: 'separator', label: ( localizeWithPath(
                        'vs/workbench/contrib/output/browser/output.contribution',
                        'extensionLogs',
                        "Extension Logs"
                    )) });
                }
                for (const { id, label } of extensionLogs) {
                    entries.push({ id, label });
                }
                const entry = await quickInputService.pick(entries, { placeHolder: ( localizeWithPath(
                    'vs/workbench/contrib/output/browser/output.contribution',
                    'selectlog',
                    "Select Log"
                )) });
                if (entry) {
                    return outputService.showChannel(entry.id);
                }
            }
        }));
    }
    registerOpenLogFileAction() {
        this._register(registerAction2(class extends Action2 {
            constructor() {
                super({
                    id: 'workbench.action.openLogFile',
                    title: ( localize2WithPath(
                        'vs/workbench/contrib/output/browser/output.contribution',
                        'openLogFile',
                        "Open Log File..."
                    )),
                    category: Categories.Developer,
                    menu: {
                        id: MenuId.CommandPalette,
                    },
                    metadata: {
                        description: 'workbench.action.openLogFile',
                        args: [{
                                name: 'logFile',
                                schema: {
                                    markdownDescription: ( localizeWithPath(
                                        'vs/workbench/contrib/output/browser/output.contribution',
                                        'logFile',
                                        "The id of the log file to open, for example `\"window\"`. Currently the best way to get this is to get the ID by checking the `workbench.action.output.show.<id>` commands"
                                    )),
                                    type: 'string'
                                }
                            }]
                    },
                });
            }
            async run(accessor, args) {
                const outputService = accessor.get(IOutputService);
                const quickInputService = accessor.get(IQuickInputService);
                const editorService = accessor.get(IEditorService);
                const fileConfigurationService = accessor.get(IFilesConfigurationService);
                const entries = ( outputService.getChannelDescriptors().filter(c => c.file && c.log)
                    .map(channel => ({ id: channel.id, label: channel.label, channel })));
                const argName = args && typeof args === 'string' ? args : undefined;
                let entry;
                if (argName) {
                    entry = entries.find(e => e.id === argName);
                }
                if (!entry) {
                    entry = await quickInputService.pick(entries, { placeHolder: ( localizeWithPath(
                        'vs/workbench/contrib/output/browser/output.contribution',
                        'selectlogFile',
                        "Select Log File"
                    )) });
                }
                if (entry) {
                    const resource = assertIsDefined(entry.channel.file);
                    await fileConfigurationService.updateReadonly(resource, true);
                    await editorService.openEditor({
                        resource,
                        options: {
                            pinned: true,
                        }
                    });
                }
            }
        }));
    }
};
OutputContribution = ( __decorate([
    ( __param(0, IOutputService)),
    ( __param(1, IEditorService)),
    ( __param(2, IFilesConfigurationService))
], OutputContribution));
( Registry.as(Extensions$1.Workbench)).registerWorkbenchContribution(OutputContribution, 3 );
( Registry.as(Extensions$2.Configuration)).registerConfiguration({
    id: 'output',
    order: 30,
    title: ( localizeWithPath(
        'vs/workbench/contrib/output/browser/output.contribution',
        'output',
        "Output"
    )),
    type: 'object',
    properties: {
        'output.smartScroll.enabled': {
            type: 'boolean',
            description: ( localizeWithPath(
                'vs/workbench/contrib/output/browser/output.contribution',
                'output.smartScroll.enabled',
                "Enable/disable the ability of smart scrolling in the output view. Smart scrolling allows you to lock scrolling automatically when you click in the output view and unlocks when you click in the last line."
            )),
            default: true,
            scope: 3 ,
            tags: ['output']
        }
    }
});
